/*
    Copyright (c) 2003, Stephane Sudre
	All rights reserved.

	Redistribution and use in source and binary forms, with or without modification, are permitted
    provided that the following conditions are met:

	Redistributions of source code must retain the above copyright notice, this list of conditions
    and the following disclaimer.

	Redistributions in binary form must reproduce the above copyright notice, this list of conditions
    and the following disclaimer in the documentation and/or other materials provided with the distribution.

	Neither the name of the WhiteBox nor the names of its contributors may be used to endorse 
    or promote products derived from this software without specific prior written permission.


	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR
    IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
    AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
    CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
    DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
    WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
    ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "AKPServerPane2Controller.h"
#import "CVSProcessEngine.h"
#import "CVSUserFormatter.h"
#import "CVSPasswordFormatter.h"

#define BUFFER_C		512

@implementation NSDictionary (sortByUsers)

- (NSComparisonResult) sortByUsers:(NSDictionary *) aDictionary
{
    return [[self objectForKey:@"Short Name"] compare:[aDictionary objectForKey:@"Short Name"] options:NSCaseInsensitiveSearch];
}

@end

@implementation AKPServerPane2Controller

- (void) awakeFromNib
{
    users_=[[NSMutableArray alloc] initWithCapacity:10];
    
    [IBedit_ setEnabled:NO];
    [IBremove_ setEnabled:NO];
}

- (BOOL) checkPaneValuesWithEngine:(id) inEngine
{
    [inEngine setUsersArray:users_];
    
    return YES;
}

- (void) processWithEngine:(id) inEngine
{
    [inEngine setUsersArray:users_];
}

- (IBAction)endDialog:(id)sender
{
    if (sender==IBOKbutton_)
    {
        NSString * tLoginName;
        NSString * tPassword;
        NSString * tPasswordVerify;
        NSDictionary * tDictionary;
        const char * tCString;
        CFIndex i,tCount;
        
        tLoginName=[IBshortName_ stringValue];
        
        // Check the length of the fields
        
        if ([tLoginName length]<=3)
        {
            NSBeep();
            
            NSRunAlertPanel(NSLocalizedStringFromTable(@"The short name you entered is too short.", @"CVSUsers", @"No comment"),NSLocalizedStringFromTable(@"The short name must be at least 4 characters.", @"CVSUsers", @"No comment"),nil,nil,nil);
            
            return;
        }
        
        // Check that this User account does not already exist on the System
        
        tCString=[tLoginName cString];
        
        // Check that this User account does not already exist in our new user list
        
        tCount=[users_ count];
        
        for(i=0;i<tCount;i++)
        {
            if ([tLoginName isEqualToString:[[users_ objectAtIndex:i] objectForKey:@"Short Name"]]==YES)
            {
                NSBeep();
            
                NSRunAlertPanel(NSLocalizedStringFromTable(@"The short name you entered is already used for another account.", @"CVSUsers", @"No comment"),NSLocalizedStringFromTable(@"You need to enter a different short name.", @"CVSUsers", @"No comment"),nil,nil,nil);
            
                return;
            }
        }
        
        // Check that the password and verify fields are the same
        
        tPassword=[IBpassword_ stringValue];
        tPasswordVerify=[IBverify_ stringValue];
        
        if ([tPassword isEqualToString:tPasswordVerify]==NO)
        {
            NSBeep();
            
            NSRunAlertPanel(NSLocalizedStringFromTable(@"The password and verified password aren't the same.", @"CVSUsers", @"No comment"), NSLocalizedStringFromTable(@"Please retype the password.", @"CVSUsers", @"No comment"), nil, nil, nil);
            
            return;
        }
        
        if ([tPassword length]==0)
        {
            NSBeep();
            
            NSRunAlertPanel(NSLocalizedStringFromTable(@"The password can't be empty.", @"CVSUsers", @"No comment"), NSLocalizedStringFromTable(@"Please set a password.", @"CVSUsers", @"No comment"), nil, nil, nil);
            
            return;
        }
        
        // Depending on the mode, either add or modify the user info to/in the array
        
        tDictionary=[NSDictionary dictionaryWithObjectsAndKeys:tLoginName,@"Short Name",tPassword,@"Password",nil];
        
        [users_ addObject:tDictionary];
        
        [IBarray_ reloadData];
        
        // Sort the array
        
        [users_ sortUsingSelector:@selector(sortByUsers:)];
        
        // Select the new user if needed
        
        [IBarray_ deselectAll:self];
        
        // A COMPLETER
        
    }
    else
    {
    }
    
    [NSApp endSheet:IBwindow_];
    [IBwindow_ orderOut:self];
}

// ---------------------------------------------------

- (void)control:(NSControl *)control didFailToValidatePartialString:(NSString *)string errorDescription:(NSString *)error
{
    if (IBshortName_==control)
    {
        if (error!=nil)
        {
            NSBeep();
        }
    }
    else
    if (IBpassword_==control || IBverify_==control)
    {
        NSBeep();
    }
}

- (void) showNewAccountSheet:(NSDictionary *) inUserSettings
{
    // Set the Formatter if needed
    
    if ([IBshortName_ formatter]==nil)
    {
        [IBshortName_ setFormatter:[[CVSUserFormatter new] autorelease]];
        [IBpassword_ setFormatter:[[CVSPasswordFormatter new] autorelease]];
        [IBverify_ setFormatter:[[CVSPasswordFormatter new] autorelease]];
    }
    
    // Set the fields if needed
    
    if (inUserSettings!=nil)
    {
        [IBshortName_ setStringValue:[inUserSettings objectForKey:@"Short Name"]];
        [IBpassword_ setStringValue:[inUserSettings objectForKey:@"Password"]];
        [IBverify_ setStringValue:[inUserSettings objectForKey:@"Password"]];
    }
    else
    {
        [IBshortName_ setStringValue:@""];
        [IBpassword_ setStringValue:@""];
        [IBverify_ setStringValue:@""];
    }
    
    [NSApp beginSheet:IBwindow_
       modalForWindow:[IBarray_ window]
        modalDelegate:nil
       didEndSelector:nil
          contextInfo:NULL];
}

- (IBAction)add:(id)sender
{
    [self showNewAccountSheet:nil];
}

- (IBAction)edit:(id)sender
{
    [self showNewAccountSheet:[users_ objectAtIndex:[IBarray_ selectedRow]]];
}

- (IBAction)remove:(id)sender
{
    NSString * errString;
    
    if ([IBarray_ numberOfSelectedRows]==1)
    {
        errString=NSLocalizedStringFromTable(@"Do you really want to remove this user?", @"CVSUsers", @"No comment");
    }
    else
    {
        errString=NSLocalizedStringFromTable(@"Do you really want to remove these users?", @"CVSUsers", @"No comment");
    }
    
    NSBeginAlertSheet(errString,
                      NSLocalizedStringFromTable(@"Remove", @"CVSUsers", @"No comment"),
                      NSLocalizedStringFromTable(@"Cancel", @"CVSUsers", @"No comment"),
                      nil,
                      [IBarray_ window],
                      self,
                      @selector(removeSheetDidEnd:returnCode:contextInfo:),
                      nil,
                      NULL,
                      NSLocalizedStringFromTable(@"This can not be undone", @"CVSUsers", @"No comment"));
}

- (void) removeSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *) contextInfo
{
    NSEnumerator * enumerator;
    NSArray * tArray;
    CFIndex i,tCount;
    CFIndex tIndex;
    
    enumerator=[IBarray_ selectedRowEnumerator];
    
    tArray=[enumerator allObjects];
    
    tCount=[tArray count];
    
    for(i=tCount-1;i>=0;i--)
    {
        tIndex=[[tArray objectAtIndex:i] intValue];
        
        [users_ removeObjectAtIndex:tIndex];
    }
    
    // Unselect All
    
    [IBarray_ deselectAll:self];
    
    // Reload Data
    
    [IBarray_ reloadData];
}

// ---------------------------------------------------

- (int )numberOfRowsInTableView:(NSTableView *)aTableView
{
    if (users_!=nil)
    {
        return [users_ count];
    }
    
    return 0;
}

- (id) tableView:(NSTableView *)aTableView objectValueForTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex
{
    if (aTableView==IBarray_)
    {
        if ([[aTableColumn identifier] isEqualToString:@"Short Name"])
        {
            NSDictionary * tDictionary;
            
            tDictionary=[users_ objectAtIndex:rowIndex];
            
            if (tDictionary!=nil)
            {
                return [tDictionary objectForKey:@"Short Name"];
            }
        }
    }
    
    return nil;
}

- (void)tableViewSelectionDidChange:(NSNotification *)aNotification
{
    if ([IBarray_ numberOfSelectedRows]==0)
    {
        [IBedit_ setEnabled:NO];
        [IBremove_ setEnabled:NO];
    }
    else
    {
        if ([IBarray_ numberOfSelectedRows]==1)
        {
            [IBedit_ setEnabled:YES];
        }
        else
        {
            [IBedit_ setEnabled:NO];
        }
        
        [IBremove_ setEnabled:YES];
    }
}

@end
