/*
    Copyright (c) 2003, Stephane Sudre
	All rights reserved.

	Redistribution and use in source and binary forms, with or without modification, are permitted
    provided that the following conditions are met:

	Redistributions of source code must retain the above copyright notice, this list of conditions
    and the following disclaimer.

	Redistributions in binary form must reproduce the above copyright notice, this list of conditions
    and the following disclaimer in the documentation and/or other materials provided with the distribution.

	Neither the name of the WhiteBox nor the names of its contributors may be used to endorse 
    or promote products derived from this software without specific prior written permission.


	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR
    IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
    AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
    CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
    DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
    WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
    ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "AKServerCVSRepositoryController.h"
#import "CVSProcessEngine.h"
#import "CVSPathFormatter.h"
#import "CVSFileNameFormatter.h"

#define CVS_DEFAULT_PATH	@"/Users/Shared/"
#define CVS_DEFAULT_NAME	@"CVS"

@implementation AKServerCVSRepositoryController

- (void) awakeFromNib
{
    NSFileManager * fileManager=[NSFileManager defaultManager];
    BOOL isDirectory;
    
    if ([fileManager fileExistsAtPath:CVS_DEFAULT_PATH isDirectory:&isDirectory]==YES && isDirectory==YES)
    {
        [IBrepositoryLocation_ setStringValue:CVS_DEFAULT_PATH];
    }
    else
    {
        // A COMPLETER
    }
    
    [IBrepositoryName_ setStringValue:CVS_DEFAULT_NAME];
    
    [IBrepositoryLocation_ setFormatter:[[CVSPathFormatter new] autorelease]];
    [IBrepositoryName_ setFormatter:[[CVSFileNameFormatter new] autorelease]];
}


- (void)selectRepositoryDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void  *)contextInfo;
{
    if (returnCode == NSOKButton)
    {
        NSOpenPanel *oPanel = (NSOpenPanel *) sheet;
        NSString *aFile = [oPanel filename];
        
        [IBrepositoryLocation_ setStringValue:aFile];
        [IBrepositoryLocation_ setNeedsDisplay:YES];
    }
}

- (void)control:(NSControl *)control didFailToValidatePartialString:(NSString *)string errorDescription:(NSString *)error
{
    if (IBrepositoryLocation_==control || IBrepositoryName_==control)
    {
        NSBeep();
    }
}

- (IBAction) select:(id)sender
{
    NSOpenPanel *oPanel = [NSOpenPanel openPanel];
    
    [oPanel setCanChooseDirectories:YES];
    [oPanel setCanChooseFiles:NO];
    [oPanel setPrompt:NSLocalizedString(@"Select", @"No comment")];
    
    [oPanel setDelegate:self];
    
    [oPanel beginSheetForDirectory:[IBrepositoryLocation_ stringValue]
                              file:nil
                    modalForWindow:[IBrepositoryLocation_ window]
                     modalDelegate:self
                    didEndSelector:@selector(selectRepositoryDidEnd:returnCode:contextInfo:)
                       contextInfo:NULL];
}

- (BOOL) checkPaneValuesWithEngine:(id) inEngine
{
    NSFileManager * tFilemanager=[NSFileManager defaultManager];
    
    if ([tFilemanager fileExistsAtPath:[IBrepositoryLocation_ stringValue]]==NO)
    {
        NSBeginAlertSheet(NSLocalizedStringFromTable(@"Invalid CVS Repository", @"CVSRepository", @"No comment"),
                          nil,
                          nil,
                          nil,
                          [IBrepositoryLocation_ window],
                          self,
                          nil,
                          nil,
                          NULL,
                          NSLocalizedStringFromTable(@"The path you entered is not valid.", @"CVSRepository", @"No comment"));
        
        return NO;
    }
    
    [inEngine setRepository:[[IBrepositoryLocation_ stringValue] stringByAppendingPathComponent:[IBrepositoryName_ stringValue]]];
    
    return YES;
}

- (void) processWithEngine:(id) inEngine
{
    [inEngine setRepository:[[IBrepositoryLocation_ stringValue] stringByAppendingPathComponent:[IBrepositoryName_ stringValue]]];
}

- (BOOL)panel:(id)sender shouldShowFilename:(NSString *)filename
{
    NSRange tRange;
    
    tRange=[filename rangeOfString:@" "];
    
    return (tRange.location==NSNotFound);
}


@end
