/*
    Copyright (c) 2003, Stephane Sudre
	All rights reserved.

	Redistribution and use in source and binary forms, with or without modification, are permitted
    provided that the following conditions are met:

	Redistributions of source code must retain the above copyright notice, this list of conditions
    and the following disclaimer.

	Redistributions in binary form must reproduce the above copyright notice, this list of conditions
    and the following disclaimer in the documentation and/or other materials provided with the distribution.

	Neither the name of the WhiteBox nor the names of its contributors may be used to endorse 
    or promote products derived from this software without specific prior written permission.


	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR
    IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
    AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
    CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
    DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
    WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
    ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "CVSProcessEngine.h"

@implementation CVSProcessEngine

- (AuthorizationRights) authorizationRights
{
    AuthorizationItem * item;
    AuthorizationRights rights;
    int i;
    char * commandPath[1] = {
                                NULL
                            };
    
    item=(AuthorizationItem *) calloc(1,sizeof(AuthorizationItem));
    
    commandPath[0]=(char *) [[[NSBundle mainBundle] pathForResource:@"setupcvspserver" ofType:@"sh"] fileSystemRepresentation];
    
    for(i=0;i<1;i++)
    {
        item[i].name = kAuthorizationRightExecute;
        item[i].value = commandPath[i];
        item[i].valueLength = strlen(commandPath[i]);
        item[i].flags = 0;
    }
    
    rights.count=1;
    rights.items = item;
    
    return rights;
}

- (int) serverMode
{
    return serverMode_;
}

- (int) userAccountMode
{
    return userAccountMode_;
}

- (NSArray *) usersArray
{
    return [[usersArray_ retain] autorelease];
}

- (NSString *) repository
{
    return [[repository_ retain] autorelease];
}

- (void) setServerMode:(int) inServerMode
{
    serverMode_=inServerMode;
}

- (void) setUserAccountMode:(int) inUserAccountMode
{
    userAccountMode_=inUserAccountMode;
}

- (void) setRepository:(NSString *) inRepository
{
    if (repository_!=nil)
    {
        [repository_ release];
    }
    
    repository_=[inRepository copy];
}

- (void) setUsersArray:(NSArray *) inUsersArray
{
    if (usersArray_!=nil)
    {
        [usersArray_ release];
    }
    
    usersArray_=[inUsersArray copy];
}

- (void) startProcess
{
    /* Overload this method */
}

- (BOOL) endProcess
{
    BOOL returnCode=NO;
    
    switch(serverMode_)
    {
        case CVS_PSERVER:
            returnCode=[self processPServer];
            break;
        case CVS_SSH_SERVER:
            returnCode=[self processSSHServer];
            break;
    }
    
    if (returnCode==YES)
    {
        NSRunAlertPanel(NSLocalizedString(@"Your CVS server has been successfully configured.", @"No comment"),NSLocalizedString(@"You can now import your project in the CVS Repository.", @"No comment"),nil,nil,nil);
    }
    else
    {
        NSBeep();
        
        NSRunAlertPanel(NSLocalizedString(@"The configuration failed.", @"No comment"),NSLocalizedString(@"Your CVS server has not been configured.", @"No comment"),nil,nil,nil);
    }
    
    return returnCode;
}

- (BOOL) processPServer
{    
    NSString * scriptPath;
    char* args[10];
    OSStatus err = 0;
    
    // Create the CVS accounts ?
        
    if (userAccountMode_==CVS_CVS_ACCOUNTS)
    {
        if (usersArray_!=nil)
        {
            int tUsersCount;
            
            tUsersCount=[usersArray_ count];
            
            if (tUsersCount>0)
            {
                int i;
                NSMutableString * tPasswordFile;
                
                tPasswordFile=[NSMutableString stringWithCapacity:256];
                
                for(i=0;i<tUsersCount;i++)
                {
                    NSDictionary * tDictionary;
                    
                    tDictionary=[usersArray_ objectAtIndex:i];
                    
                    if (i>0)
                    {
                        [tPasswordFile appendString:@"\n"];
                    }
                    
                    [tPasswordFile appendString:[tDictionary objectForKey:@"Short Name"]];
                    [tPasswordFile appendString:@"\t"];
                    [tPasswordFile appendString:[tDictionary objectForKey:@"Password"]];
                }
                
                [tPasswordFile writeToFile:@"/tmp/cvsaccounts.txt" atomically:YES];
            }
        }
    }
    
    // Create the CVS Repository and set-up NetInfo, inetd and all the stuff
    
    args[0] = "-d";
    args[1] = (char *) [repository_ fileSystemRepresentation];
    args[2] = NULL;
    
    scriptPath=[[NSBundle mainBundle] pathForResource:@"setupcvspserver" ofType:@"sh"];
    
    err = AuthorizationExecuteWithPrivileges(authorizationRef_,
                                             [scriptPath fileSystemRepresentation],
                                             0, args, NULL);

    return (err==errAuthorizationSuccess);
}

- (BOOL) processSSHServer
{
    // TO BE CONTINUED
    
    return NO;
}

@end
