#!/bin/sh
#
# This script prepares a machine to be a CVS server.
#
# Usage: makeMachineCVSServer
#
# Example: makeMachineCVSServer /Local/Public/cvs/repository
#
# IMPORTANT:
#
# - You must run this script as Administrator (root).
#
# - Run this script ONLY ONCE.
#
# - The cvs repository path must not contains any spaces.
#

# Look for parameters

while getopts d: opt
do
	case  $opt in
	d) CVS_DIR=$OPTARG;;
	?) echo "Usage : create_base [-d base_directory]";;
	esac
done

# Check the parameters

if [ -z $CVS_DIR ] ; then
	CVS_DIR="/Users/Shared/CVS";
fi

# Creation of the repository

/bin/mkdir -p $CVS_DIR
cd $CVS_DIR

# Creation of the base

/usr/bin/cvs -d $CVS_DIR init

# Add the user accounts if needed

if [ -f "/tmp/cvsaccounts.txt" ] ; then
    /usr/bin/awk -v PATH=$CVS_DIR '{ print "/usr/bin/htpasswd",( NR == 1 ? "-c -b" : "-b" ), PATH"/CVSROOT/passwd",$(NF - 1), $NF }' "/tmp/cvsaccounts.txt" | /bin/sh
    
    /bin/rm /tmp/cvsaccounts.txt
fi

# Create an entry for the cvspserver service in the local
# NetInfo domain. Map it to port 2401, protocol tcp

NB=`/usr/bin/niutil -list . /services | /usr/bin/grep cvspserver | wc -l`

if [ $NB = 0 ] ; then
	/usr/bin/niutil -create . /services/cvspserver
    /usr/bin/niutil -createprop . /services/cvspserver port 2401
    /usr/bin/niutil -createprop . /services/cvspserver protocol tcp
fi

# Append a cvspserver entry into /etc/inetd.conf

NB=`/usr/bin/grep -v "#" /etc/inetd.conf | /usr/bin/grep cvspserver | wc -l`

if [ $NB = 0 ] ; then
    echo >> /etc/inetd.conf
    echo -n "cvspserver stream tcp nowait root /usr/bin/cvs cvs --allow-root=$CVS_DIR pserver" >> /etc/inetd.conf

    # Send HUP signal to the inetd process to cause it to
    # re-read the configuration file.

    kill -s HUP `/bin/ps cax | /usr/bin/grep inetd | /usr/bin/awk '{ print $1 }'`
fi

